SystemOrganization addCategory: #'MonticelloFileTree-Core'!

!MCMethodDefinition methodsFor: '*monticellofiletree-core' stamp: 'dkh 2/2/2012 15:31'!
setTimeStamp: aString
    ^ timeStamp := aString! !

MCWriter subclass: #MCFileTreeStWriter
	instanceVariableNames: 'stWriter initializers orderedClassNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Core'!

!MCFileTreeStWriter class methodsFor: 'accessing' stamp: 'ezra 11/1/2011 17:52'!
readerClass
	^ MCStReader! !

!MCFileTreeStWriter methodsFor: 'initialize-release' stamp: 'dkh 2/2/2012 16:18'!
initialize
    stWriter := MCFileTreePackageStructureStWriter new
        initializers: (initializers := Set new);
        yourself.
    orderedClassNames := OrderedCollection new! !

!MCFileTreeStWriter methodsFor: 'visiting' stamp: 'dkh 2/1/2012 20:01'!
visitClassDefinition: definition
	orderedClassNames add: definition className.
	self
		writeInDirectoryName: definition className
		fileName: definition className
		extension: '.st'
		visit: [ 
			stWriter writeClassDefinition: definition.
			definition hasClassInstanceVariables
				ifTrue: [ stWriter writeMetaclassDefinition: definition ].
			definition hasComment
				ifTrue: [ stWriter writeClassComment: definition ] ]! !

!MCFileTreeStWriter methodsFor: 'visiting' stamp: 'dkh 1/30/2012 16:30'!
visitClassTraitDefinition: definition
    self flag: #'skippedForNow'! !

!MCFileTreeStWriter methodsFor: 'visiting' stamp: 'dkh 2/1/2012 20:02'!
visitMetaclassDefinition: definition
	self
		writeInDirectoryName: definition className , '_class'
		fileName: definition className
		extension: '.st'
		visit: [ stWriter writeMetaclassDefinition: definition ]! !

!MCFileTreeStWriter methodsFor: 'visiting' stamp: 'dkh 2/2/2012 16:32'!
visitMethodDefinition: definition
    | filename directoryname |
    directoryname := definition classIsMeta
        ifTrue: [ definition className , '_class' ]
        ifFalse: [ definition className ].
    filename := definition selector == #'/'
        ifTrue: [ 'encoded slash' ]
        ifFalse: [ definition selector ].
    self
        writeInDirectoryName: directoryname
        fileName: filename
        extension: '.st'
        visit: [ stWriter visitMethodDefinition: definition ].
    self
        writeInDirectoryName: directoryname
        fileName: filename
        extension: '.meta'
        visit: [ stWriter writeMethodMetadata: definition ]! !

!MCFileTreeStWriter methodsFor: 'visiting' stamp: 'dkh 2/1/2012 20:02'!
visitOrganizationDefinition: defintion
	self
		writeInDirectoryName: '.'
		fileName: 'categories'
		extension: '.st'
		visit: [ defintion categories do: [ :cat | stWriter writeCategory: cat ] ]! !

!MCFileTreeStWriter methodsFor: 'visiting' stamp: 'dkh 1/30/2012 16:30'!
visitScriptDefinition: definition
    self flag: #'skippedForNow'! !

!MCFileTreeStWriter methodsFor: 'visiting' stamp: 'dkh 1/30/2012 16:31'!
visitTraitDefinition: definition
    self flag: #'skippedForNow'! !

!MCFileTreeStWriter methodsFor: 'initialize-release' stamp: 'dkh 2/2/2012 14:52'!
writeConfigFile
    self
        writeInDirectoryName: '.'
        fileName: ''
        extension: '.filetree'
        visit: [ stWriter writeSignature ]! !

!MCFileTreeStWriter methodsFor: 'initialize-release' stamp: 'dkh 2/2/2012 16:24'!
writeDefinitions: aCollection
    "the correct initialization order is unknown if some classes are missing in the image"

    [ 
    self writeConfigFile.
    stWriter acceptVisitor: self forDefinitions: aCollection.
    self writeInitializers ]
        on: Error
        do: [ :ex | ex halt ]! !

!MCFileTreeStWriter methodsFor: 'visiting' stamp: 'dkh 2/1/2012 20:14'!
writeInDirectoryName: directoryName fileName: fileName extension: ext visit: visitBlock
	| directory |
	directory := directoryName = '.'
		ifTrue: [ stream packageFileDirectory ]
		ifFalse: [ stream subPackageFileDirectoryFor: directoryName ].
	directory
		forceNewFileNamed: fileName , ext
		do: [ :file | 
			file lineEndConvention: #lf.
			stWriter stream: file.
			visitBlock value ]! !

!MCFileTreeStWriter methodsFor: 'initialize-release' stamp: 'dkh 2/1/2012 19:55'!
writeInitializers
	self
		writeInDirectoryName: '.'
		fileName: 'initializers'
		extension: '.st'
		visit: [ 
			stWriter
				writePresentInitializers;
				writeAbsentInitializers ] ! !

!DirectoryEntryFile methodsFor: '*monticellofiletree-core' stamp: 'ezra 11/1/2011 17:55'!
readStreamDo: aBlock
	^ directory readOnlyFileNamed: self name do: aBlock! !

MCMczWriter subclass: #MCFileTreeWriter
	instanceVariableNames: 'directory packageFileDirectory cachedPackageFileDirectoryEntries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Core'!

!MCFileTreeWriter class methodsFor: 'accessing' stamp: 'OttoBehrens 10/24/2011 20:35'!
readerClass
	^ MCFileTreeReader! !

!MCFileTreeWriter methodsFor: 'writing' stamp: 'galatians 05/13/2005 14:15'!
addString: string at: path
	string isEmpty
		ifFalse: [ 
			self packageFileDirectory forceNewFileNamed: path do: [:file | file nextPutAll: string]] ! !

!MCFileTreeWriter methodsFor: 'writing' stamp: 'OttoBehrens 10/24/2011 21:34'!
flush
	! !

!MCFileTreeWriter methodsFor: 'visiting' stamp: 'ezra 10/25/2011 16:53'!
initializePackageFileDirectoryCache
	cachedPackageFileDirectoryEntries := Dictionary new.
	self packageFileDirectory entries
		do: [ :entry | 
			entry isDirectory
				ifTrue: [ cachedPackageFileDirectoryEntries at: entry name put: entry asFileDirectory ] ]! !

!MCFileTreeWriter methodsFor: 'writing' stamp: 'ezra 10/25/2011 17:10'!
packageFileDirectory
	^ packageFileDirectory := stream directory directoryNamed: directory! !

!MCFileTreeWriter methodsFor: 'serializing' stamp: 'revelations 11/9/2011 13:56'!
serializeVersionInfo: aVersionInfo
	^ String streamContents: [ :s | self writeVersionInfo: aVersionInfo onStream: s ]! !

!MCFileTreeWriter methodsFor: 'accessing' stamp: 'OttoBehrens 10/24/2011 20:40'!
snapshotWriterClass
	^ MCFileTreeStWriter! !

!MCFileTreeWriter methodsFor: 'visiting' stamp: 'ezra 10/25/2011 17:29'!
subPackageFileDirectoryFor: directoryName
	^ cachedPackageFileDirectoryEntries
		at: directoryName
		ifAbsentPut: [ 
			(self packageFileDirectory)
				createDirectory: directoryName;
				directoryNamed: directoryName ]! !

!MCFileTreeWriter methodsFor: 'visiting' stamp: 'OttoBehrens 10/25/2011 10:15'!
writeSnapshot: aSnapshot
	(self snapshotWriterClass on: self) writeDefinitions: aSnapshot definitions! !

!MCFileTreeWriter methodsFor: 'visiting' stamp: 'dkh 1/30/2012 20:57'!
writeVersion: aVersion
	directory := (MCFileTreeRepository parseName: aVersion info name) last.
	self packageFileDirectory exists
		ifTrue: [ self packageFileDirectory recursiveDelete ].
	self packageFileDirectory assureExistence.
	self initializePackageFileDirectoryCache.
	super writeVersion: aVersion! !

!MCFileTreeWriter methodsFor: 'serializing' stamp: 'revelations 11/9/2011 14:18'!
writeVersionInfo: aVersionInfo onStream: s
	s nextPut: $(.
	#(#name #message #id #date #time #author)
		do: [ :sel | 
			s
				nextPutAll: sel;
				space.
			((aVersionInfo perform: sel) ifNil: [ '' ]) asString printOn: s.
			s lf ].
	s
		nextPutAll: 'ancestors ()';
		lf;
		nextPutAll: 'stepChildren ())'! !

!MCRepository methodsFor: '*monticellofiletree-core' stamp: 'dkh 2/1/2012 13:53'!
retrieveVersionsWithPackageNames: packageNames
    ^ self readableFileNames
        collect: [ :each | 
            | name |
            name := (each copyUpToLast: $.) copyUpTo: $(.
            name last isDigit
                ifFalse: [ 
                    Array
                        with: name
                        with: ''
                        with: ''
                        with: each ]
                ifTrue: [ 
                    Array
                        with: (packageNames add: (name copyUpToLast: $-))
                        with: ((name copyAfterLast: $-) copyUpTo: $.)
                        with: ((name copyAfterLast: $-) copyAfter: $.) asInteger
                        with: each	"pkg name"	"user"	"version" ] ]! !

MCDirectoryRepository subclass: #MCFileTreeRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Core'!

!MCFileTreeRepository class methodsFor: 'instance creation' stamp: 'dkh 1/31/2012 14:13'!
description
    ^ 'filetree://'! !

!MCFileTreeRepository class methodsFor: 'accessing' stamp: 'dkh 1/31/2012 14:16'!
packageExtension
    ^ '.tree'! !

!MCFileTreeRepository class methodsFor: 'utility' stamp: 'dkh 1/31/2012 14:17'!
parseName: aString
    ^ self parseName: aString extension: self packageExtension! !

!MCFileTreeRepository class methodsFor: 'utility' stamp: 'dkh 1/31/2012 14:14'!
parseName: aString extension: extension
    "picked up from GoferVersionReference>>parseName:"

    | info basicName package branch author versionNumber packageName |
    basicName := aString last isDigit
        ifTrue: [ aString ]
        ifFalse: [ (aString copyUpToLast: $.) copyUpTo: $( ].
    package := basicName copyUpToLast: $-.
    (package includes: $.)
        ifFalse: [ branch := '' ]
        ifTrue: [ 
            branch := '.' , (package copyAfter: $.).
            package := package copyUpTo: $. ].
    author := (basicName copyAfterLast: $-) copyUpToLast: $..
    versionNumber := (basicName copyAfterLast: $-) copyAfterLast: $..
    (versionNumber notEmpty and: [ versionNumber allSatisfy: [ :each | each isDigit ] ])
        ifTrue: [ versionNumber := versionNumber asNumber ]
        ifFalse: [ versionNumber := 0 ].
    packageName := package , branch.
    ^ {packageName.
    author.
    versionNumber.
    (packageName , extension)}! !

!MCFileTreeRepository methodsFor: 'as yet unclassified' stamp: 'ezra 10/28/2011 19:24'!
allFileNames
	^(directory entries select: [:entry | entry isDirectory and: [self canReadFileNamed: entry name]]) collect: [:entry | entry name]! !

!MCFileTreeRepository methodsFor: 'as yet unclassified' stamp: 'ezra 10/28/2011 18:41'!
allFileNamesForVersionNamed: aString
	^ {(self treeDirectoryNameForVersionNamed: aString)}! !

!MCFileTreeRepository methodsFor: 'as yet unclassified' stamp: 'OttoBehrens 10/25/2011 07:21'!
basicStoreVersion: aVersion
	MCFileTreeWriter fileOut: aVersion on: self! !

!MCFileTreeRepository methodsFor: 'as yet unclassified' stamp: 'OttoBehrens 10/25/2011 07:20'!
cachedFileNames
	^ #()! !

!MCFileTreeRepository methodsFor: 'as yet unclassified' stamp: 'dkh 1/31/2012 14:13'!
canReadFileNamed: aString
    ^ aString endsWith: self packageExtension! !

!MCFileTreeRepository methodsFor: 'descriptions' stamp: 'dkh 1/31/2012 14:19'!
description
    ^ self class description , super description! !

!MCFileTreeRepository methodsFor: 'accessing' stamp: 'revelations 11/2/2011 12:07'!
directoryFrom: aDirectory
	self directory: (FileDirectory on: aDirectory)! !

!MCFileTreeRepository methodsFor: 'actions' stamp: 'OttoBehrens 10/25/2011 08:57'!
fileDirectoryOn: fileName
	^ FileDirectory on: (directory fullNameFor: fileName)! !

!MCFileTreeRepository methodsFor: 'as yet unclassified' stamp: 'ezra 10/28/2011 18:46'!
goferVersionFrom: aVersionReference
	^ self loadVersionFromFileNamed: (self treeDirectoryNameForVersionNamed: aVersionReference name)! !

!MCFileTreeRepository methodsFor: 'actions' stamp: 'dkh 1/31/2012 14:14'!
packageDescriptionFromPackageDirectory: packageDirectory
    | info |
    info := self versionInfoForPackageDirectory: packageDirectory.
    ^ self parseName: info name! !

!MCFileTreeRepository methodsFor: 'actions' stamp: 'ezra 10/28/2011 18:32'!
packageDescriptionsFromReadableFileNames
	^ ((self readableFileNames collect: [ :fileName | self fileDirectoryOn: fileName ])
		select: [ :packageDirectory | packageDirectory exists ])
		collect: [ :packageDirectory | self packageDescriptionFromPackageDirectory: packageDirectory ]! !

!MCFileTreeRepository methodsFor: 'private' stamp: 'dkh 1/31/2012 14:17'!
packageExtension
    ^ self class packageExtension! !

!MCFileTreeRepository methodsFor: 'private' stamp: 'dkh 1/31/2012 14:17'!
parseName: aString
    ^ self class parseName: aString! !

!MCFileTreeRepository methodsFor: 'i/o' stamp: 'ezra 10/28/2011 18:53'!
readStreamForFileNamed: aString do: aBlock
	^ aBlock value: directory! !

!MCFileTreeRepository methodsFor: 'as yet unclassified' stamp: 'dkh 1/31/2012 14:14'!
treeDirectoryNameForVersionNamed: aString
    ^ (self parseName: aString) last! !

!MCFileTreeRepository methodsFor: 'as yet unclassified' stamp: 'OttoBehrens 10/25/2011 07:20'!
versionFromFileNamed: aString
	^ self loadVersionFromFileNamed: aString! !

!MCFileTreeRepository methodsFor: 'actions' stamp: 'OttoBehrens 10/25/2011 09:00'!
versionInfoForPackageDirectory: packageDirectory
	^ (MCFileTreeReader on: packageDirectory containingDirectory fileName: packageDirectory localName)
		loadVersionInfo;
		info! !

!MCFileTreeRepository methodsFor: 'as yet unclassified' stamp: 'OttoBehrens 10/25/2011 07:22'!
versionInfoFromFileNamed: aString
	^ self loadVersionInfoFromFileNamed: aString! !

!MCFileTreeRepository methodsFor: 'as yet unclassified' stamp: 'ezra 10/28/2011 18:36'!
versionNameFromFileName: aString
	| description |
	description := self packageDescriptionFromPackageDirectory: (self fileDirectoryOn: aString).
	^ description first , '-' , description second , '.' , description third printString! !

!MCFileTreeRepository methodsFor: 'interface' stamp: 'OttoBehrens 10/26/2011 20:13'!
versionWithInfo: aVersionInfo ifAbsent: errorBlock
	(self allFileNamesForVersionNamed: aVersionInfo name)
		ifNotEmpty: [ :aCollection | ^ self versionFromFileNamed: aCollection first ].
	^ errorBlock value! !

!MCFileTreeRepository methodsFor: 'i/o' stamp: 'OttoBehrens 10/26/2011 20:45'!
writeStreamForFileNamed: aString replace: aBoolean do: aBlock
	self error: 'we do not open a single stream, but write multiple files'! !

!MCFileRepositoryInspector methodsFor: '*monticellofiletree-core' stamp: 'OttoBehrens 10/24/2011 23:31'!
refresh
	| packageNames |
	packageNames := Set new.
	packageList := nil.
	versions := repository retrieveVersionsWithPackageNames: packageNames.
	newer := Set new.
	inherited := Set new.
	loaded := Set new.
	MCWorkingCopy allManagers
		do: [ :each | 
			| latest |
			each ancestors
				do: [ :ancestor | 
					loaded add: ancestor name.
					ancestor
						ancestorsDoWhileTrue: [ :heir | 
							(inherited includes: heir name)
								ifTrue: [ false ]
								ifFalse: [ 
									inherited add: heir name.
									true ] ] ].
			latest := (versions select: [ :v | v first = each package name ]) detectMax: [ :v | v third ].
			(latest notNil
				and: [ 
					each ancestors
						allSatisfy: [ :ancestor | 
							| av |
							av := ((ancestor name copyAfterLast: $-) copyAfter: $.) asInteger.
							av < latest third
								or: [ av = latest third and: [ ((ancestor name copyAfterLast: $-) copyUpTo: $.) ~= latest second ] ] ] ])
				ifTrue: [ newer add: each package name ] ].	"		select: [ :each | packageNames includes: each packageName]"
	self
		changed: #packageList;
		changed: #versionList! !

MCMczReader subclass: #MCFileTreeReader
	instanceVariableNames: 'packageDirectory fileTreeStructureVersion'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Core'!

!MCFileTreeReader class methodsFor: 'accessing' stamp: 'OttoBehrens 10/24/2011 22:49'!
extension
	^ 'tree'! !

!MCFileTreeReader class methodsFor: 'reading' stamp: 'OttoBehrens 10/25/2011 00:58'!
on: s fileName: f
	^ (self on: s)
		packageDirectory: f;
		yourself! !

!MCFileTreeReader methodsFor: 'utilities' stamp: 'dkh 2/6/2012 17:08'!
addClassAndMethodDefinitionsFromDirectoryEntries: entries
    entries
        do: [ :element | 
            element isDirectory
                ifTrue: [ 
                    | directory |
                    directory := packageDirectory directoryNamed: element name.
                    (self separateMethodMetaAndSource
                        ifTrue: [ directory entries select: [ :entry | entry name endsWith: '.st' ] ]
                        ifFalse: [ directory entries ])
                        do: [ :file | 
                            | methodDefinition |
                            (methodDefinition := self definitionFromFile: file inDirectory: directory)
                                ifNotNil: [ 
                                    (self separateMethodMetaAndSource and: [ methodDefinition isMethodDefinition ])
                                        ifTrue: [ 
                                            directory
                                                fileNamed: methodDefinition selector asString , '.meta'
                                                do: [ :fileStream | methodDefinition setTimeStamp: (Author fixStamp: fileStream contents) ] ].
                                    definitions add: methodDefinition ] ] ] ]! !

!MCFileTreeReader methodsFor: 'utilities' stamp: 'dkh 2/2/2012 15:48'!
addDefinitionFromFile: directoryEntry inDirectory: aDirectory
    (self definitionFromFile: directoryEntry inDirectory: aDirectory) ifNotNil: [ :def | definitions add: def ]! !

!MCFileTreeReader methodsFor: 'accessing' stamp: 'OttoBehrens 10/25/2011 09:31'!
basicVersion
	^ (MCVersion new)
		setPackage: self package
			info: self info
			snapshot: self snapshot
			dependencies: self dependencies;
		yourself! !

!MCFileTreeReader methodsFor: 'utilities' stamp: 'dkh 2/2/2012 15:57'!
definitionFromFile: directoryEntry inDirectory: aDirectory
    | defs reader |
    directoryEntry ifNil: [ ^ nil ].
    aDirectory
        fileNamed: directoryEntry name
        do: [ :fileStream | 
            | methodDefinition |
            reader := MCStReader on: fileStream.
            (defs := reader definitions) size <= 1
                ifFalse: [ 'we are attempting to write exactly 1 definition per file' halt ] ].
    defs size = 0
        ifTrue: [ ^ nil ].
    ^ defs first! !

!MCFileTreeReader methodsFor: 'utilities' stamp: 'dkh 2/2/2012 16:04'!
loadDefinitions
    | entries |
    definitions := OrderedCollection new.
    entries := packageDirectory entries.
    [ 
    fileTreeStructureVersion := #'0.0'.
    (entries detect: [ :entry | entry name = '.filetree' ] ifNone: [  ])
        ifNotNil: [ :configEntry | configEntry readStreamDo: [ :fileStream | fileTreeStructureVersion := fileStream contents asSymbol ] ].
    self
        addDefinitionFromFile: (entries detect: [ :entry | entry name beginsWith: 'categories' ] ifNone: [  ])
            inDirectory: packageDirectory;
        addClassAndMethodDefinitionsFromDirectoryEntries: entries;
        addDefinitionFromFile: (entries detect: [ :entry | entry name beginsWith: 'initializers' ] ifNone: [  ])
            inDirectory: packageDirectory ]
        on: Error
        do: [ :error | error halt ]! !

!MCFileTreeReader methodsFor: 'utilities' stamp: 'OttoBehrens 10/25/2011 01:31'!
loadDependencies
	dependencies := Array new! !

!MCFileTreeReader methodsFor: 'accessing' stamp: 'revelations 11/2/2011 13:27'!
packageDirectory: aByteString
	packageDirectory := stream directoryNamed: aByteString! !

!MCFileTreeReader methodsFor: 'utilities' stamp: 'revelations 11/9/2011 11:26'!
parseMember: fileName
	| tokens contents |
	contents := packageDirectory fileNamed: fileName do: [ :file | file contents ].
	tokens := (self scanner scanTokens: contents) first.
	^ self associate: tokens! !

!MCFileTreeReader methodsFor: 'testing' stamp: 'dkh 2/2/2012 15:18'!
separateMethodMetaAndSource
    ^ fileTreeStructureVersion == #'0.1'! !

!MCMczReader methodsFor: '*monticellofiletree-core' stamp: 'OttoBehrens 10/25/2011 01:11'!
extractDefinitionsFrom: member
	self extractDefinitionsFromFile: member contentStream named: member fileName! !

!MCMczReader methodsFor: '*monticellofiletree-core' stamp: 'OttoBehrens 10/25/2011 01:12'!
extractDefinitionsFromFile: file named: fileName
	(MCSnapshotReader readerClassForFileNamed: fileName)
		ifNotNil: [ :rc | 
			| reader |
			reader := rc on: file text.
			definitions addAll: reader definitions ]! !

MCStWriter subclass: #MCFileTreePackageStructureStWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloFileTree-Core'!

!MCFileTreePackageStructureStWriter methodsFor: '*monticellofiletree-core' stamp: 'dkh 2/2/2012 15:14'!
structureSignature
    "method meta data stored in separate file from method source"

    ^ '0.1'! !

!MCFileTreePackageStructureStWriter methodsFor: '*monticellofiletree-core' stamp: 'dkh 2/2/2012 16:46'!
writeMethodMetadata: definition
    stream nextPutAll: definition timeStamp asString! !

!MCFileTreePackageStructureStWriter methodsFor: 'writing' stamp: 'dkh 2/2/2012 16:09'!
writeMethodPreamble: definition
    stream
        cr;
        nextPut: $!!;
        nextPutAll: definition fullClassName;
        nextPutAll: ' methodsFor: ';
        nextPutAll: definition category asString printString;
        nextPutAll: '!!';
        cr! !

!MCStWriter methodsFor: '*monticellofiletree-core' stamp: 'ezra 10/26/2011 17:25'!
absentInitializers
	^ initializers reject: [ :each | Smalltalk hasClassNamed: each key ]! !

!MCStWriter methodsFor: '*monticellofiletree-core' stamp: 'ezra 10/26/2011 17:33'!
acceptVisitor: aVisitor forDefinitions: aCollection
	(MCDependencySorter sortItems: aCollection)
		do: [ :ea | ea accept: aVisitor ]
		displayingProgress: 'Writing definitions...'! !

!MCStWriter methodsFor: '*monticellofiletree-core' stamp: 'revelations 11/3/2011 14:51'!
initializers: aCollection
	initializers := aCollection! !

!MCStWriter methodsFor: '*monticellofiletree-core' stamp: 'ezra 10/26/2011 17:19'!
presentInitializers
	^ initializers select: [ :each | Smalltalk hasClassNamed: each key ]! !

!MCStWriter methodsFor: '*monticellofiletree-core' stamp: 'dkh 2/2/2012 15:14'!
structureSignature
    ^ '0.0'! !

!MCStWriter methodsFor: '*monticellofiletree-core' stamp: 'ezra 10/26/2011 17:25'!
writeAbsentInitializers
	(self absentInitializers asSortedCollection: [ :a :b | a key <= b key ])
		do: [ :association | stream nextPutAll: association value contents ]! !

!MCStWriter methodsFor: '*monticellofiletree-core' stamp: 'ezra 10/25/2011 16:05'!
writeClassDefinition: definition
	self chunkContents: [ :s | definition printDefinitionOn: s ]! !

!MCStWriter methodsFor: '*monticellofiletree-core' stamp: 'ezra 10/26/2011 17:35'!
writeDefinitions: aCollection
	"the correct initialization order is unknown if some classes are missing in the image"

	initializers := Set new.
	self
		acceptVisitor: self forDefinitions: aCollection;
		writePresentInitializers;
		writeAbsentInitializers! !

!MCStWriter methodsFor: '*monticellofiletree-core' stamp: 'dkh 2/2/2012 16:16'!
writeMethodMetadata: definition
    "nothing to write here"

    ! !

!MCStWriter methodsFor: '*monticellofiletree-core' stamp: 'ezra 10/26/2011 17:21'!
writePresentInitializers
	| orderedClasses presentInitializers |
	presentInitializers := self presentInitializers.
	orderedClasses := (Class
		superclassOrder: (presentInitializers collect: [ :each | Smalltalk classOrTraitNamed: each key ]))
		collect: [ :each | each name ].
	orderedClasses
		do: [ :className | stream nextPutAll: (presentInitializers detect: [ :each | each key = className ]) value contents ]! !

!MCStWriter methodsFor: '*monticellofiletree-core' stamp: 'dkh 2/2/2012 15:04'!
writeSignature
    stream nextPutAll: self structureSignature! !

!MCVersionInfo methodsFor: '*monticellofiletree-core' stamp: 'revelations 11/3/2011 12:27'!
versionNumber
	^ ((self name copyAfterLast: $-) copyAfter: $.) asInteger ifNil: [ 0 ]! !

!MCFileBasedRepository methodsFor: '*monticellofiletree-core' stamp: 'ezra 10/28/2011 18:25'!
packageDescriptionsFromReadableFileNames
	^ self readableFileNames
		collect: [ :each | 
			| name |
			name := (each copyUpToLast: $.) copyUpTo: $(.
			name last isDigit
				ifFalse: [ 
					{name.
					''.
					''.
					each} ]
				ifTrue: [ 
					| packageName author versionNumber |
					packageName := name copyUpToLast: $-.
					author := (name copyAfterLast: $-) copyUpTo: $..
					versionNumber := ((name copyAfterLast: $-) copyAfter: $.) asInteger ifNil: [ 0 ].
					{packageName.
					author.
					versionNumber.
					each} ] ]! !

!MCFileBasedRepository methodsFor: '*monticellofiletree-core' stamp: 'ezra 10/28/2011 18:25'!
retrieveVersionsWithPackageNames: packageNames
	| packageDescriptions |
	packageDescriptions := self packageDescriptionsFromReadableFileNames.
	packageNames addAll: (packageDescriptions collect: [ :packageDescription | packageDescription first ]).
	^ packageDescriptions select: [ :each | (each at: 3) isNumber ]! !
